@@ -21,8 +21,9 @@ def sanitize_k8s_name(name,
                       allow_capital_underscore=False,
                       allow_dot=False,
                       allow_slash=False,
-                      max_length=63,
-                      suffix_space=0):
+                      max_length=57,
+                      suffix_space=0,
+                      rev_truncate=False):
     """From _make_kubernetes_name
       sanitize_k8s_name cleans and converts the names in the workflow.
 
@@ -36,7 +37,7 @@ def sanitize_k8s_name(name,
         in this name (i.e. for parameters)
       allow_dot: whether to allow dots in this name (i.e. for labels)
       allow_slash: whether to allow slash in this name (i.e. for label and annotation keys)
-      max_length: maximum length of K8s name, default: 63
+      max_length: maximum length of K8s name is 63, but KFP will append 6 chars as uuid. Default: 57
       suffix_space: number of characters reserved for a suffix to be appended
 
     Returns:
@@ -58,7 +59,11 @@ def sanitize_k8s_name(name,
 
     # truncate if length exceeds max_length
     max_length = max_length - suffix_space
-    k8s_name = k8s_name[:max_length].rstrip('-') if len(k8s_name) > max_length else k8s_name
+    if len(k8s_name) > max_length:
+      if rev_truncate:
+        k8s_name = k8s_name[len(k8s_name) - max_length:].strip('-')
+      else:
+        k8s_name = k8s_name[:max_length].rstrip('-')
 
     return k8s_name
 
