@@ -16,6 +16,7 @@ import json
 import re
 import yaml
 import logging
+import hashlib
 
 from collections import OrderedDict
 from typing import List, Text, Dict, Any
@@ -32,6 +33,11 @@ from .. import __version__
 RESOURCE_OP_IMAGE = ":".join(["aipipeline/kubectl-wrapper", __version__])
 TEKTON_HOME_RESULT_PATH = "/tekton/home/tep-results/"
 
+# The image to use in basic bash steps such as copying results in multi-step.
+TEKTON_BASH_STEP_IMAGE = 'busybox'
+TEKTON_COPY_RESULTS_STEP_IMAGE = 'library/bash'
+GENERATE_COMPONENT_SPEC_ANNOTATIONS = True
+
 
 def _get_base_step(name: str):
     """Base image step for running bash commands.
@@ -45,7 +51,7 @@ def _get_base_step(name: str):
         Dict[Text, Any]
     """
     return {
-        'image': 'busybox',
+        'image': TEKTON_BASH_STEP_IMAGE,
         'name': name,
         'script': '#!/bin/sh\nset -exo pipefail\n'
     }
@@ -75,7 +81,7 @@ def _get_copy_result_step_template(step_number: int, result_maps: list):
         "name": "copy-results-%s" % str(step_number),
         "args": args,
         "command": ["sh", "-c"],
-        "image": "library/bash"
+        "image": TEKTON_COPY_RESULTS_STEP_IMAGE
     }
 
 
@@ -526,9 +532,17 @@ def _op_to_template(op: BaseOp,
                                                                             for volume in processed_op.volumes]
         template['spec']['volumes'].sort(key=lambda x: x['name'])
 
-    if isinstance(op, dsl.ContainerOp) and op._metadata:
-        template.setdefault('metadata', {}).setdefault('annotations', {})['pipelines.kubeflow.org/component_spec'] = \
-            json.dumps(op._metadata.to_dict(), sort_keys=True)
+    if isinstance(op, dsl.ContainerOp) and op._metadata and GENERATE_COMPONENT_SPEC_ANNOTATIONS:
+        component_spec_dict = op._metadata.to_dict()
+        component_spec_digest = hashlib.sha256(json.dumps(component_spec_dict, sort_keys=True).encode()).hexdigest()
+        component_name = component_spec_dict.get('name', op.name)
+        component_version = component_name + '@sha256=' + component_spec_digest
+        digested_component_spec_dict = {'name': component_name,
+                                        'outputs': component_spec_dict.get('outputs', []),
+                                        'version': component_version
+        }
+        template.setdefault('metadata', {}).setdefault('annotations', {})['pipelines.kubeflow.org/component_spec_digest'] = \
+            json.dumps(digested_component_spec_dict, sort_keys=True)
 
     if isinstance(op, dsl.ContainerOp) and op.execution_options:
         if op.execution_options.caching_strategy.max_cache_staleness:
@@ -613,6 +627,6 @@ def _op_to_template(op: BaseOp,
         # Update actifact item location to the latest stage in order to properly track and store all the artifacts.
         for i, artifact in enumerate(artifact_items[op.name]):
             if artifact[0] not in verified_result_size_map[step_counter].keys():
-                artifact[1] = '%s%s' % (TEKTON_HOME_RESULT_PATH, artifact[0])
+                artifact[1] = '%s%s' % (TEKTON_HOME_RESULT_PATH, sanitize_k8s_name(artifact[0]))
                 artifact_items[op.name][i] = artifact
     return template
