@@ -26,6 +26,7 @@ from distutils.util import strtobool
 import collections
 from os import environ as env
 from typing import Callable, List, Text, Dict, Any
+import hashlib
 
 import yaml
 # Kubeflow Pipeline imports
@@ -37,10 +38,10 @@ from kfp.dsl._for_loop import LoopArguments
 from kfp.dsl._metadata import _extract_pipeline_metadata
 # KFP-Tekton imports
 from kfp_tekton.compiler import __tekton_api_version__ as tekton_api_version
-from kfp_tekton.compiler._data_passing_rewriter import fix_big_data_passing
+from kfp_tekton.compiler._data_passing_rewriter import fix_big_data_passing, BIG_DATA_PATH_FORMAT
 from kfp_tekton.compiler._k8s_helper import convert_k8s_obj_to_json, sanitize_k8s_name, sanitize_k8s_object
 from kfp_tekton.compiler._op_to_template import _op_to_template
-from kfp_tekton.compiler._tekton_handler import _handle_tekton_pipeline_variables, _handle_tekton_custom_task
+from kfp_tekton.compiler._tekton_handler import _handle_tekton_pipeline_variables, _handle_tekton_custom_task, _process_argo_vars
 from kfp_tekton.compiler.pipeline_utils import TektonPipelineConf
 from kfp_tekton.compiler.yaml_utils import dump_yaml
 from kfp_tekton.tekton import TEKTON_CUSTOM_TASK_IMAGES, DEFAULT_CONDITION_OUTPUT_KEYWORD, LOOP_PIPELINE_NAME_LENGTH, LOOP_GROUP_NAME_LENGTH
@@ -51,6 +52,8 @@ DEFAULT_ARTIFACT_ENDPOINT_SCHEME = env.get('DEFAULT_ARTIFACT_ENDPOINT_SCHEME', '
 TEKTON_GLOBAL_DEFAULT_TIMEOUT = strtobool(env.get('TEKTON_GLOBAL_DEFAULT_TIMEOUT', 'false'))
 # DISABLE_CEL_CONDITION should be True until CEL is officially merged into Tekton main API.
 DISABLE_CEL_CONDITION = True
+# Default timeout is one year
+DEFAULT_TIMEOUT_MINUTES = "525600m"
 
 
 def _get_super_condition_template():
@@ -120,6 +123,7 @@ class TektonCompiler(Compiler):
     # Input and output artifacts are hash maps for metadata tracking.
     # artifact_items is the artifact dependency map
     # loops_pipeline recorde the loop tasks information for each loops
+    # produce_taskspec Produces task spec as part of Tekton pipelineRuns
     self.input_artifacts = {}
     self.output_artifacts = {}
     self.artifact_items = {}
@@ -132,6 +136,7 @@ class TektonCompiler(Compiler):
     self.pipeline_annotations = {}
     self.tekton_inline_spec = True
     self.resource_in_separate_yaml = False
+    self.produce_taskspec = True
     super().__init__(**kwargs)
 
   def _set_pipeline_conf(self, tekton_pipeline_conf: TektonPipelineConf):
@@ -713,8 +718,14 @@ class TektonCompiler(Compiler):
                         return True
               return False
             if is_custom_task_output(condition_operand1) or is_custom_task_output(condition_operand2):
-              map_cel_vars = lambda a: '$(tasks.%s.results.%s)' % (sanitize_k8s_name(a['value'].split('.')[-1]),
-                sanitize_k8s_name(a['output_name'])) if a.get('type', '') == dsl.PipelineParam else a.get('value', '')
+              def map_cel_vars(a):
+                if a.get('type', '') == dsl.PipelineParam:
+                  op_name = sanitize_k8s_name(a['op_name'])
+                  output_name = sanitize_k8s_name(a['output_name'])
+                  return '$(tasks.%s.results.%s)' % (op_name, output_name)
+                else:
+                  return a.get('value', '')
+
               condition_refs[template['metadata']['name']] = [
                   {
                     'input': map_cel_vars(condition_operand1),
@@ -751,17 +762,19 @@ class TektonCompiler(Compiler):
             for key, value in custom_task_args.items():
               if key not in non_param_keys:
                 task_params.append({'name': key, 'value': value})
+            task_orig_params = task_ref['params']
             task_ref = {
               'name': template['metadata']['name'],
               'params': task_params,
               # For processing Tekton parameter mapping later on.
-              'orig_params': task_ref['params'],
+              'orig_params': task_orig_params,
               'taskRef': {
                 'name': custom_task_args['name'],
                 'apiVersion': custom_task_args['apiVersion'],
                 'kind': custom_task_args['kind']
               }
             }
+
             # Only one of --taskRef and --taskSpec allowed.
             if custom_task_args.get('taskRef', '') and custom_task_args.get('taskSpec', ''):
               raise("Custom task invalid configuration %s, Only one of --taskRef and --taskSpec allowed." % custom_task_args)
@@ -790,8 +803,7 @@ class TektonCompiler(Compiler):
                 task_ref = {
                   'name': template['metadata']['name'],
                   'params': task_params,
-                  # For processing Tekton parameter mapping later on.
-                  'orig_params': task_ref['params'],
+                  'orig_params': task_orig_params,
                   'taskSpec': {
                     'apiVersion': custom_task_args['apiVersion'],
                     'kind': custom_task_args['kind'],
@@ -808,16 +820,18 @@ class TektonCompiler(Compiler):
         if task_ref.get('taskSpec', ''):
           task_ref['taskSpec']['metadata'] = task_ref['taskSpec'].get('metadata', {})
           task_labels = template['metadata'].get('labels', {})
-          task_ref['taskSpec']['metadata']['labels'] = task_labels
           task_labels['pipelines.kubeflow.org/pipelinename'] = task_labels.get('pipelines.kubeflow.org/pipelinename', '')
           task_labels['pipelines.kubeflow.org/generation'] = task_labels.get('pipelines.kubeflow.org/generation', '')
           cache_default = self.pipeline_labels.get('pipelines.kubeflow.org/cache_enabled', 'true')
           task_labels['pipelines.kubeflow.org/cache_enabled'] = task_labels.get('pipelines.kubeflow.org/cache_enabled', cache_default)
 
           task_annotations = template['metadata'].get('annotations', {})
-          task_ref['taskSpec']['metadata']['annotations'] = task_annotations
           task_annotations['tekton.dev/template'] = task_annotations.get('tekton.dev/template', '')
 
+          # Updata default metadata at the end.
+          task_ref['taskSpec']['metadata']['labels'] = task_labels
+          task_ref['taskSpec']['metadata']['annotations'] = task_annotations
+
         task_refs.append(task_ref)
 
     # process input parameters from upstream tasks for conditions and pair conditions with their ancestor conditions
@@ -878,6 +892,40 @@ class TektonCompiler(Compiler):
       if parent_group:
         if condition_refs.get(parent_group[-2], []):
           task['when'] = condition_refs.get(op_name_to_parent_groups[task['name']][-2], [])
+          # Travser the rest of the parent indices to check whether there are nested when conditions
+          depended_conditions = []
+          # flatten condition ref to list
+          condition_task_refs_temp = []
+          for condition_task_ref in condition_task_refs.values():
+            for ref in condition_task_ref:
+              if not string_condition_refs.get(ref['name'], False):
+                condition_task_refs_temp.append(ref)
+
+          # Get depended parent when expression
+          def get_when_task(input_task_when, depended_conditions):
+            when_task_name = re.findall('\$\(tasks.([^ \t\n.:,;{}]+).results.([^ \t\n.:,;{}]+)\)', input_task_when)
+            if when_task_name:
+              for when_task in task_refs + condition_task_refs_temp:
+                if when_task['name'] == when_task_name[0][0]:
+                  if when_task.get('when', []):
+                    for when_dependent in when_task['when']:
+                      if when_dependent.get("input", ""):
+                        depended_conditions.append(when_dependent.get("input", ""))
+          get_when_task(task['when'][0].get("input", ""), depended_conditions)
+          parent_index = -3
+          while abs(parent_index) <= len(op_name_to_parent_groups[task['name']]):
+            if 'condition-' in op_name_to_parent_groups[task['name']][parent_index]:
+              # If the nested when conditions already have parent when dependency, then skip
+              for when_exp in condition_refs.get(op_name_to_parent_groups[task['name']][parent_index], []):
+                get_when_task(when_exp.get("input", ""), depended_conditions)
+                if when_exp.get("input", ""):
+                  if when_exp.get("input", "") not in depended_conditions:
+                    task['when'].append(when_exp)
+                else:
+                  task['when'].append(when_exp)
+              parent_index -= 1
+            else:
+              break
       if op != None and op.dependent_names:
         task['runAfter'] = op.dependent_names
 
@@ -974,11 +1022,10 @@ class TektonCompiler(Compiler):
     # add timeout params to task_refs, instead of task.
     for task in task_refs:
       op = pipeline.ops.get(task['name'])
-      if not (task.get('taskRef', '') != '' or task.get('taskSpec', '') != '' and 'apiVersion' in task['taskSpec']):
-        if op != None and (not TEKTON_GLOBAL_DEFAULT_TIMEOUT or op.timeout):
-          task['timeout'] = '%ds' % op.timeout
-      elif op.timeout > 0:  # i.e. incase of custom task, don't set timeout field by default.
+      if op.timeout > 0:
         task['timeout'] = '%ds' % op.timeout
+      else:
+        task['timeout'] = DEFAULT_TIMEOUT_MINUTES
 
     # handle resourceOp cases in pipeline
     self._process_resourceOp(task_refs, pipeline)
@@ -1034,6 +1081,9 @@ class TektonCompiler(Compiler):
       # Remove pipeline level label for 'pipelines.kubeflow.org/cache_enabled' as it overwrites task level label
       pipeline_run['metadata']['labels'].pop('pipelines.kubeflow.org/cache_enabled', None)
 
+    # Add big data passing path format
+    self.pipeline_annotations['pipelines.kubeflow.org/big_data_passing_format'] = BIG_DATA_PATH_FORMAT
+
     if self.pipeline_annotations:
       pipeline_run['metadata']['annotations'] = pipeline_run['metadata'].setdefault('annotations', {})
       pipeline_run['metadata']['annotations'].update(self.pipeline_annotations)
@@ -1066,7 +1116,10 @@ class TektonCompiler(Compiler):
 
     # add workflow level timeout to pipeline run
     if not TEKTON_GLOBAL_DEFAULT_TIMEOUT or pipeline.conf.timeout:
-      pipeline_run['spec']['timeout'] = '%ds' % pipeline.conf.timeout
+      if pipeline.conf.timeout > 0:
+        pipeline_run['spec']['timeout'] = '%ds' % pipeline.conf.timeout
+      else:
+        pipeline_run['spec']['timeout'] = DEFAULT_TIMEOUT_MINUTES
 
     # generate the Tekton podTemplate for image pull secret
     if len(pipeline.conf.image_pull_secrets) > 0:
@@ -1207,7 +1260,7 @@ class TektonCompiler(Compiler):
         pipeline_conf,
     )
 
-    workflow = fix_big_data_passing(workflow)
+    workflow = fix_big_data_passing(workflow, self.loops_pipeline, '-'.join(self._group_names[:-1] + [""]))
 
     workflow.setdefault('metadata', {}).setdefault('annotations', {})['pipelines.kubeflow.org/pipeline_spec'] = \
       json.dumps(pipeline_meta.to_dict(), sort_keys=True)
@@ -1250,46 +1303,21 @@ class TektonCompiler(Compiler):
     """Dump pipeline workflow into yaml spec and write out in the format specified by the user.
 
     Args:
-      workflow: Workflow spec of the pipline, dict.
+      workflow: Workflow spec of the pipeline, dict.
       package_path: file path to be written. If not specified, a yaml_text string
         will be returned.
     """
-    yaml_text = dump_yaml(workflow)
 
-    # Use regex to replace all the Argo variables to Tekton variables. For variables that are unique to Argo,
-    # we raise an Error to alert users about the unsupported variables. Here is the list of Argo variables.
-    # https://github.com/argoproj/argo/blob/master/docs/variables.md
-    # Since Argo variables can be used in anywhere in the yaml, we need to dump and then parse the whole yaml
-    # using regular expression.
-    tekton_var_regex_rules = [
-        {
-          'argo_rule': '{{inputs.parameters.([^ \t\n.:,;{}]+)}}',
-          'tekton_rule': '$(inputs.params.\g<1>)'
-        },
-        {
-          'argo_rule': '{{outputs.parameters.([^ \t\n.:,;{}]+).path}}',
-          'tekton_rule': '$(results.\g<1>.path)'
-        },
-        {
-          'argo_rule': '{{workflow.uid}}',
-          'tekton_rule': '$(context.pipelineRun.uid)'
-        },
-        {
-          'argo_rule': '{{workflow.name}}',
-          'tekton_rule': '$(context.pipelineRun.name)'
-        },
-        {
-          'argo_rule': '{{workflow.namespace}}',
-          'tekton_rule': '$(context.pipelineRun.namespace)'
-        },
-        {
-          'argo_rule': '{{workflow.parameters.([^ \t\n.:,;{}]+)}}',
-          'tekton_rule': '$(params.\g<1>)'
-        }
-    ]
-    for regex_rule in tekton_var_regex_rules:
-      yaml_text = re.sub(regex_rule['argo_rule'], regex_rule['tekton_rule'], yaml_text)
+    yaml_text = ""
+    pipeline_run = workflow
+    if pipeline_run.get("spec", {}) and pipeline_run["spec"].get("pipelineSpec", {}) and \
+      pipeline_run["spec"]["pipelineSpec"].get("tasks", []):
+      yaml_text = dump_yaml(_handle_tekton_pipeline_variables(pipeline_run))
+    else:
+      yaml_text = dump_yaml(workflow)
 
+    # Convert Argo variables to Tekton variables.
+    yaml_text = _process_argo_vars(yaml_text)
     unsupported_vars = re.findall(r"{{[^ \t\n.:,;{}]+\.[^ \t\n:,;{}]+}}", yaml_text)
     if unsupported_vars:
       raise ValueError('These Argo variables are not supported in Tekton Pipeline: %s' % ", ".join(str(v) for v in set(unsupported_vars)))
@@ -1299,11 +1327,6 @@ class TektonCompiler(Compiler):
           'Please create a new issue at https://github.com/kubeflow/kfp-tekton/issues '
           'attaching the pipeline DSL code and the pipeline YAML.')
 
-    pipeline_run = yaml.load(yaml_text, Loader=yaml.FullLoader)
-    if pipeline_run.get("spec", {}) and pipeline_run["spec"].get("pipelineSpec", {}) and \
-      pipeline_run["spec"]["pipelineSpec"].get("tasks", []):
-      yaml_text = dump_yaml(_handle_tekton_pipeline_variables(pipeline_run))
-
     if package_path is None:
       return yaml_text
 
@@ -1328,24 +1351,31 @@ class TektonCompiler(Compiler):
           'The output path %s should end with one of the following formats: '
           '[.tar.gz, .tgz, .zip, .yaml, .yml]' % package_path)
 
-  def _create_and_write_workflow(self,
-                                 pipeline_func: Callable,
-                                 pipeline_name: Text = None,
-                                 pipeline_description: Text = None,
-                                 params_list: List[dsl.PipelineParam] = None,
-                                 pipeline_conf: dsl.PipelineConf = None,
-                                 package_path: Text = None,
-                                 ) -> None:
-    """Compile the given pipeline function and dump it to specified file format."""
+  def prepare_workflow(self,
+                       pipeline_func: Callable,
+                       pipeline_name: Text = None,
+                       pipeline_description: Text = None,
+                       params_list: List[dsl.PipelineParam] = None,
+                       pipeline_conf: dsl.PipelineConf = None,
+                       ):
+    """Compile the given pipeline function and return a python Dict."""
+
     workflow = self._create_workflow(
-        pipeline_func,
-        pipeline_name,
-        pipeline_description,
-        params_list,
-        pipeline_conf)
+      pipeline_func,
+      pipeline_name,
+      pipeline_description,
+      params_list,
+      pipeline_conf)
+
     # Separate loop workflow from the main workflow
+    pipeline_loop_crs = []
     if self.loops_pipeline:
       pipeline_loop_crs, workflow = _handle_tekton_custom_task(self.loops_pipeline, workflow, self.recursive_tasks, self._group_names)
+      if workflow['spec'].get('workspaces', []):
+        for pipeline_loop_cr in pipeline_loop_crs:
+          pipeline_loop_cr['spec']['workspaces'] = workflow['spec'].get('workspaces', [])
+          pipeline_loop_cr['spec']['pipelineSpec']['workspaces'] = [
+            {'name': workspace['name']} for workspace in workflow['spec'].get('workspaces', [])]
       inlined_as_taskSpec: List[Text] = []
       recursive_tasks_names: List[Text] = [x['taskRef'].get('name', "") for x in self.recursive_tasks]
       if self.tekton_inline_spec:
@@ -1369,25 +1399,81 @@ class TektonCompiler(Compiler):
         # Preserve order of params, required by tests.
         if 'params' in workflow['spec']:
           workflow['spec']['params'] = sorted(workflow['spec']['params'], key=lambda kv: (kv['name']))
-      TektonCompiler._write_workflow(workflow=workflow, package_path=package_path)
+        pipeline_loop_crs = [cr for cr in pipeline_loop_crs if cr['metadata'].get("name", "") not in inlined_as_taskSpec]
+    return pipeline_loop_crs, workflow
+
+  def _create_and_write_workflow(self,
+                                 pipeline_func: Callable,
+                                 pipeline_name: Text = None,
+                                 pipeline_description: Text = None,
+                                 params_list: List[dsl.PipelineParam] = None,
+                                 pipeline_conf: dsl.PipelineConf = None,
+                                 package_path: Text = None,
+                                 ) -> None:
+    """Compile the given pipeline function and dump it to specified file format."""
+
+    pipeline_loop_crs, workflow = self.prepare_workflow(
+      pipeline_func,
+      pipeline_name,
+      pipeline_description,
+      params_list,
+      pipeline_conf)
+
+    # create cr yaml for only those pipelineLoop cr which could not be converted to inlined spec.
+    loop_package_annotations = []
+    for i in range(len(pipeline_loop_crs)):
+      if pipeline_loop_crs[i]['metadata'].get('name', ""):
+        if self.resource_in_separate_yaml:
+          TektonCompiler._write_workflow(workflow=pipeline_loop_crs[i],
+                                         package_path=os.path.splitext(package_path)[0] +
+                                                      "_pipelineloop_cr" + str(i + 1) + '.yaml')
+        else:
+          pipeline_loop_cr = TektonCompiler._write_workflow(workflow=collections.OrderedDict(pipeline_loop_crs[i]))
+          loop_package_annotations.append(yaml.load(pipeline_loop_cr, Loader=yaml.FullLoader))
+    if loop_package_annotations:
+      workflow['metadata']['annotations']['tekton.dev/resource_templates'] = json.dumps(loop_package_annotations,
+                                                                                        sort_keys=True)
+    # Need to compiles after all the CRs being processed.
+    # Convert taskspec into task templates if specified.
+    if not self.produce_taskspec:
+      component_sha = {}
+      for task in workflow['spec']['pipelineSpec']['tasks']:
+        if task.get('taskSpec'):
+          component_spec_digest = hashlib.sha1(json.dumps(task['taskSpec'], sort_keys=True).encode()).hexdigest()
+          if component_spec_digest not in component_sha.keys():
+            task_template = {}
+            task_template['metadata'] = {}
+            if task['taskSpec'].get('metadata', None):
+              task_template['metadata'] = task['taskSpec'].pop('metadata', None)
+            if task['taskSpec'].get('apiVersion', None) and task['taskSpec'].get('kind', None):
+              task_template['apiVersion'] = task['taskSpec']['apiVersion']
+              task_template['kind'] = task['taskSpec']['kind']
+            else:
+              task_template['apiVersion'] = tekton_api_version
+              task_template['kind'] = 'Task'
+            task_template['spec'] = task['taskSpec']
+            task_template['metadata']['name'] = component_spec_digest
+            component_sha[component_spec_digest] = task_template
+          task.pop("taskSpec", None)
+          task['taskRef'] = {'name': component_spec_digest}
+      # Output task templates into individual files if specified, else append task templates to annotations
+      if self.resource_in_separate_yaml:
+        for key, value in component_sha.items():
+          TektonCompiler._write_workflow(workflow=value,
+                                         package_path=os.path.splitext(package_path)[0] +
+                                         key + '.yaml')
+      else:
+        resource_templates = workflow['metadata']['annotations'].get('tekton.dev/resource_templates', [])
+        if resource_templates:
+          resource_templates = json.loads(resource_templates)
+        for value in component_sha.values():
+          resource_templates.append(value)
+        if resource_templates:
+          workflow['metadata']['annotations']['tekton.dev/resource_templates'] = json.dumps(resource_templates,
+                                                                                            sort_keys=True)
+
+    TektonCompiler._write_workflow(workflow=workflow, package_path=package_path)   # Tekton change
 
-      # create cr yaml for only those pipelineLoop cr which could not be converted to inlined spec.
-      loop_package_annotations = []
-      for i in range(len(pipeline_loop_crs)):
-        if pipeline_loop_crs[i]['metadata'].get('name', "") not in inlined_as_taskSpec:
-          if self.resource_in_separate_yaml:
-            TektonCompiler._write_workflow(workflow=pipeline_loop_crs[i],
-                                           package_path=os.path.splitext(package_path)[0] +
-                                                        "_pipelineloop_cr" + str(i + 1) + '.yaml')
-          else:
-            pipeline_loop_cr = TektonCompiler._write_workflow(workflow=collections.OrderedDict(pipeline_loop_crs[i]))
-            loop_package_annotations.append(yaml.load(pipeline_loop_cr, Loader=yaml.FullLoader))
-      if loop_package_annotations:
-        workflow['metadata']['annotations']['tekton.dev/resource_templates'] = json.dumps(loop_package_annotations, sort_keys=True)
-      # Need to compiles after all the CRs being processed.
-      TektonCompiler._write_workflow(workflow=workflow, package_path=package_path)
-    else:
-      TektonCompiler._write_workflow(workflow=workflow, package_path=package_path)   # Tekton change
     # Separate custom task CR from the main workflow
     for i in range(len(self.custom_task_crs)):
       TektonCompiler._write_workflow(workflow=self.custom_task_crs[i],
